// --> --> --> --> --> --> --> --> --> --> --> --> --> --> --> --> --> 
// Freccia.cpp : Generates score events for the arrow-storm simulator
// Created by Mathieu Bosi, 2007 A.D.
// --> --> --> --> --> --> --> --> --> --> --> --> --> --> --> --> -->

#include <math.h>
#include <stdlib.h>


#define M_PI        3.14159265358979323846


float frand(float min_val, float max_val)
{ 
float rand_k = (rand()%32000) / (32000.0-1.0);

 return min_val + rand_k*(max_val - min_val);
}


char *file_name = "Freccia.sco.inc";


int main(int argc, char* argv[])
{

	FILE *fp;
	printf("\nArrows trajectories generator! \\o/ \n\n");

	// Open the output file:
	if( (fp = fopen(file_name, "w")) == NULL)
	{
		printf("Error while creating the file %s\n", file_name);
		return -1;
	}

//------------------------------------------



// Macro to convert from Km/h to m/s
#define KMH_2_MS(x) ((float)(x)*(1000.0/3600.0))

srand(160181); // Only if we always want a particular pseudo-random sequence


int   num_events = 250;  // How many arrow events
float total_time = 30.0; // Duration in seconds

float distance_min = 10.0; // Minimum distance (meters)
float distance_max = 30.0; // Maximum distance (meters)
float speed_min = KMH_2_MS(250.0); // Minimum speed
float speed_max = KMH_2_MS(300.0); // Maximum speed

// Add the global reverb activation instrument call in the score:
fprintf(fp, "i \"Riverbero8Ch\" 0 %.3f; Reverb\n\n", total_time); 

//
// Compute each note/event:
//
for(int i=0; i < num_events; i++)
{
float action_time, duration, dB, cps, x0, y0, x1, y1;

float ktime = ((float)i / (float)(num_events-1.0));


//action_time = frand(0.0, total_time); // Uniform distribution
action_time = pow(ktime, .5) * total_time; // More events in the beginning

float dist  = frand(distance_min, distance_max);
float alpha = 4.0 * (action_time * M_PI*2.0) / total_time;

// Create two event "sources"
if(i&1) alpha = -alpha;

// Compute end and start positions
 x0 = cos(alpha) * dist;
 y0 = sin(alpha) * dist;
 x1 = cos(alpha) * 1.5 + frand(-.25, .25);
 y1 = sin(alpha) * 1.5 + frand(-.25, .25);

/*
   x0 = frand(-4.0, 4.0);
	 y0 = 8.0;//frand(.1, .3);
	 
	 float kd = frand(distance_min, distance_max) / sqrt(x0*x0 + y0*y0);
	 x0 *= kd;
	 y0 *= kd;

	 x1 = frand(-2.0, 2.0);
	 y1 = frand(-1.0, 1.0);
*/

   // Compute speed and other related parametrs
	 float speed = frand(speed_min, speed_max);
	 // duration = distance / speed
	 duration = 2.0 * sqrt( pow(x1-x0,2) + pow(y1-y0,2) ) / KMH_2_MS( speed );
	 dB = frand(60.0, 90.0);
	 cps = 1000.0 + 2000.0*((speed - speed_min) / (speed_max - speed_min));

	// FRECCIA: INSTRUMENT SCORE LAYOUT
	// p1	 2			3		4		5		6		7		8		9
	// i#  act.   dur	dB	cps	x0	y0	x1	y1

	 // write the score line for the flying arrow (instr #1)
	 fprintf(fp, "i%i\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\n",
					      1, action_time, duration, dB, cps, x0, y0, x1, y1);

	 // Randomly trigger arrows' hit/miss sounds instrument (instr #2)
   // p1	2				3			4			5				6			7
   // i#	act.		dur		ID		rate		x0		y0
	 if(rand()%100 > 70)
	 {
		static int wave_id[12] = {1,2,3,4, 11,12,13,14, 21,22,23,24};

		int rand_id = (rand()%100 > 30) ? wave_id[ rand()%4 ] : wave_id[ 4 + rand()%(12-4) ];

	  fprintf(fp, "i%i\t%.3f\t%.3f\t%i\t%.3f\t%.3f\t%.3f\n",
					  2, action_time + duration, 2.0, rand_id, frand(.7, 1.0/.7), x1, y1);
	 }


}
//------------------------------------------



// Close the score file:
if( fclose(fp) != 0)
	{
		printf("Error while closing the file %s\n", file_name);
		return -1;
	}


	return 0;
}

